#include <iostream.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <new.h>

#define SAMPLES    30
#define MIN_SIZE   50
#define MAX_SIZE   500

unsigned char *n1;
unsigned char *n2;
unsigned int RandSeedw0 = 0x8405, RandSeedw2 = 0x808;

int SIZE1, SIZE2;

int gordan(double **, int, int);
void shr(unsigned char *, int);
void subtract(unsigned char *, unsigned char *, int);
int compare(unsigned char *, unsigned char *);
void NOD(unsigned char *, unsigned char *);
unsigned my_random(unsigned number);
void fill(unsigned char *, int);
void info();
double f(int, int, int);
void my_handler();
void input(unsigned char *);
void output(unsigned char *, unsigned int);

struct data {
  int m;
  int n;
  double Tr;
  double Tcalc;
} res[SAMPLES];

ostream& operator << (ostream&, data);

int gordan(double **a, int n, int m)
{
  int i, ir;
  double s, c;

  for (int j = 0; j < n; j++)
  {
    for (s = 0, i = 0; i < n - j; i++)
      if (fabs(a[i][j]) > fabs(s))
        s = a[ir = i][j];

    if (s == 0) return 1;

    for (int k = j + 1; k < m; k++)
    {
      c = a[ir][k] / s;
      for (i = 0; i < ir; i++)
        a[i][k] -= c * a[i][j];
      for (i = ir + 1; i < n; i++)
        a[i - 1][k] = a[i][k] - c * a[i][j];
      a[n - 1][k] = c;
    }
  }

  return 0;
}

void shr(unsigned char *s, int size)
{
  for (int i = 0; i < size; i++, s++)
  {
    *s /= 2;
    if (i < size - 1) *s |= (*(s + 1) % 2) * 128;
  }
}

void shrr(unsigned char *s, int N)
{
  unsigned char *c = (unsigned char *)(new char[200]), *p = c, *d = s;
  unsigned int o = 0, i;

  for (i = 0; i < N / 8; i++)
    *c++ = 0;

  for (i = 0; i < SIZE1; i++)
  {
    *c = *s << (N % 8);
    *c += o;
    o = *s >> (8 - N % 8);
    c++;
    s++;
  }

  c = p;
  s = d;

  for (i = 0; i < N / 8 + SIZE1; i++)
    *s++ = *c++;

  *s = 0;
}

void subtract(unsigned char *s1, unsigned char *s2, int size)
{
  int o = 0, t;

  for (int i = 0; i < size; i++, s1++, s2++)
  {
    t = *s1 - *s2 - o;
    if (t < 0)
    {
      *s1 = t + 256;
      o = 1;
    }
    else
    {
      *s1 = t;
      o = 0;
    }
  }
  if (o) *s1 -= o;
}

int compare(unsigned char *s1, unsigned char *s2)
{
  int size1 = SIZE1, size2 = SIZE2, i;
  while (!s1[size1 - 1]) size1--;
  while (!s2[size2 - 1]) size2--;
  if (size1 < size2) return -1;
  if (size1 > size2) return 1;
  for (i = 0; i < size1 - 1; i++, s1++, s2++);
  for (i = 0; i < size1; i++, s1--, s2--)
  {
    if (*s1 > *s2) return 1;
    if (*s1 < *s2) return -1;
  }
  return 0;
}

void NOD(unsigned char *s1, unsigned char *s2)
{
  long N = 0;

  while (compare(s1, s2))                         // O(1)
  {                                               //          m = log a
    while (!(*s1 % 2) && !(*s2 % 2))              //          n = log b
    {                                             //
      shr(s1, SIZE1);                             // O(n)
      shr(s2, SIZE2);                             // O(m)
      N++;                                        //          m + n
    }                                             //          } E   O(n + m)
    while (!(*s1 % 2)) shr(s1, SIZE1);            // O(n)      i=0
    while (!(*s2 % 2)) shr(s2, SIZE2);            // O(m)
    int res = compare(s1, s2);                    // O(1)
    if (res < 0)                                  //
      subtract(s2, s1, SIZE1);                    // O(n)
    else if (res)                                 //
      subtract(s1, s2, SIZE2);                    // O(m)
  }
  shrr(s1, N);
}

unsigned int my_random(unsigned number)
{
  unsigned int Neww0, Neww2;

  Neww0 = RandSeedw0 * 0x8405;
  Neww2 += RandSeedw0 * 808;
  Neww2 += RandSeedw2 * 0x8405;
  Neww0++;
  RandSeedw0 = Neww2;
  RandSeedw2 = Neww0;

  return (RandSeedw2 / (0x10000 / number)) & 0x7F;
}

void fill(unsigned char *s, int size)
{
  for (int i = 0; i < size; i++, s++)
    *s = my_random(256);
}

ostream& operator << (ostream& i, data r)
{
  i << " ";
  i.width(5);
  i << r.m;
  i << "  ";
  i.width(5);
  i << r.n;
  i << "  ";
  i.width(5);
  i << r.m + r.n;
  i << "  ";
  i.width(12);
  i.precision(8);
  i << r.Tr;
  i << "  ";
  i.width(12);
  i << r.Tcalc;
  i << "  ";
  i.width(12);
  i.precision(6);
  i << fabs(r.Tcalc - r.Tr);
  i << "  \n";

  return i;
}

void info()
{
  cout << "Ŀ\n";
  cout << "     M      N    M + N       Tr           Tcalc          dT      \n";
  cout << "Ĵ\n";
  for (int i = 0; i < SAMPLES; i++)
  {
    cout << "";
    cout.width(3);
    cout << i + 1;
    cout << res[i];
  }
  cout << "\n";
}

double f(int index, int size1, int size2)
{
  switch (index)
  {
    case 0: return size1;
    case 1: return size2;
    case 2: return 1;
  }

  return 0;
}

void my_handler()
{
  cout << " 墠⠥ \n";
  abort();
}

void input(unsigned char *s)
{
  unsigned char *c = (unsigned char *)(new char[200]), *d;
  unsigned int o = 0, t, l = 1, i;

  cin >> c;

  for (i = 0; i < SIZE1; i++)
    s[i] = 0;

  for (d = c; *d; d++)
  {
    o = *d - '0';
    for (i = 0; i < l; i++)
    {
      o += s[i] * 10;
      s[i] = o % 256;
      o /= 256;
    }
    if (o)
      s[l++] = o % 256;
  }

  delete c;
}

void output(unsigned char *s, unsigned int size)
{
  unsigned char table[] = "0123456789ABCDEF";

  for (int i = 0; i < size; i++)
  {
    cout << (char)(table[s[size - i - 1] >> 4]);
    cout << (char)(table[s[size - i - 1] & 15]);
  }
  cout << "\n";
}

void main()
{
  set_new_handler(my_handler);
//  int i;

//  double times;
//  double **g = new double *[SAMPLES];

  n1 = (unsigned char *)(new char[200]);
  n2 = (unsigned char *)(new char[200]);

  SIZE1 = 20;
  SIZE2 = 20;

  input(n1);
  output(n1, 20);
  input(n2);
  output(n2, 20);
  NOD(n1, n2);
  output(n1, 20);

  delete n2;
  delete n1;
  /*cout << "Wait";
  for (i = 0; i < SAMPLES; i++)
  {
    SIZE1 = my_random(MAX_SIZE - MIN_SIZE) + MIN_SIZE;
    SIZE2 = my_random(MAX_SIZE - MIN_SIZE) + MIN_SIZE;
    res[i].m = SIZE1;
    res[i].n = SIZE2;
    n1 = (unsigned char *)(new char[SIZE1]);
    n2 = (unsigned char *)(new char[SIZE2]);
    fill(n1, SIZE1);
    fill(n2, SIZE2);
    times = clock();
    NOD(n1, n2);
    times = (double)((clock() - times) / CLOCKS_PER_SEC);
    res[i].Tr = times;
    cout << ".";
    delete n2;
    delete n1;
  }
  for (int j = 0; j < 3; j++)
  {
    g[j] = new double[4];
    g[j][3] = 0;
    for (int i = 0; i < SAMPLES; i++)
      g[j][3] += res[i].Tr * f(j, res[i].m, res[i].n);
    for (int l = 0; l < 3; l++)
    {
      g[j][l] = 0;
      for (int i = 0; i < SAMPLES; i++)
        g[j][l] += f(l, res[i].m, res[i].n) * f(j, res[i].m, res[i].n);
    }
  }
  cout << "\nGordan is: " << gordan(g, 3, 4) << "\n\n";
  for (i = 0; i < SAMPLES; i++)
  {
    res[i].Tcalc = g[0][3] * res[i].m + g[1][3] * res[i].n + g[2][3];
  }
  info();
  cout << "\n";
  for (i = 0; i < 3; i++)
  {
    cout << "c" << i + 1 << "=" << g[i][3] << "\n";
  }*/
}
